@@ -203,10 +203,10 @@ def make_with_diffusers(
         "num_images_per_prompt": num_outputs,
     }
     if init_image:
-        cmd["image"] = get_image(init_image)
+        cmd["image"] = get_image(init_image).convert("RGB")
         cmd["strength"] = prompt_strength
     if init_image_mask:
-        cmd["mask_image"] = get_image(init_image_mask)
+        cmd["mask_image"] = get_image(init_image_mask).convert("RGB")
 
     if init_image:
         operation_to_apply = "inpainting" if init_image_mask else "img2img"
@@ -241,7 +241,7 @@ def make_with_diffusers(
             cmd["prompt"] = ""
             # workaround for a bug in diffusers 0.14 and before. the actual prompt used is
             # in the prompt_embeds field, but the legacy inpainting implementation still
-            # looks for the prompt field (will submit a PR to diffusers)
+            # looks for the prompt field (pending PR: https://github.com/huggingface/diffusers/pull/2842)
     elif isinstance(operation_to_apply, StableDiffusionInpaintPipeline):
         del cmd["strength"]
 
